-- don't load if class is wrong
local _, myclass = UnitClass("player")
if myclass ~= "WARLOCK" then return end

-- EndarkenMe app
edm = {}

local LGT = LibStub:GetLibrary("LibGroupTalents-1.0", true)
if not LGT then
   print("Endarken Me is unable to load LibGroupTalents-1.0 so must exit.")
   return
end

edm.major = 0
edm.minor = 99
edm.build = "b"

-- Spell/Buff definitions
local i_darkintent 			= 80398				-- Dark Intent (spell)
local i_darkintent_b    = 85767       -- Dark Intent (buff)
local n_darkintent 			= GetSpellInfo(i_darkintent)
local b_di = GetSpellInfo(i_darkintent_b)
local di_t = 0
local di_refresh_t = 10 * 60 * 1000        -- refresh when less than 10 minutes

-------------------------------------
-- Classes (non-localized)
-------------------------------------
local n_HUNTER				= "HUNTER"
local n_PALADIN       = "PALADIN"
local n_DRUID         = "DRUID"
local n_DEATHKNIGHT 	= "DEATHKNIGHT"
local n_ROGUE         = "ROGUE"
local n_SHAMAN        = "SHAMAN"
local n_WARRIOR       = "WARRIOR"
local n_WARLOCK       = "WARLOCK"
local n_PRIEST        = "PRIEST"
local n_MAGE          = "MAGE"

-------------------------------------
-- Talents by name (localized)
-------------------------------------
local n_Druid_talent_Balance      	= "Balance"
local n_Druid_talent_FeralCombat  	= "Feral Combat"
local n_Druid_talent_Restoration  	= "Restoration"
                                  	
local n_DeathKnight_talent_Blood  	= "Blood"
local n_DeathKnight_talent_Frost  	= "Frost"
local n_DeathKnight_talent_Unholy 	= "Unholy"
                                  	
local n_Paladin_talent_Holy       	= "Holy"
local n_Paladin_talent_Protection 	= "Protection"
local n_Paladin_talent_Retribution	= "Retribution"
                                  	
local n_Shaman_talent_Elemental   	= "Elemental"
local n_Shaman_talent_Enhancement 	= "Enhancement"
local n_Shaman_talent_Restoration 	= "Restoration"
                                  	
local n_Hunter_talent_BeastMastery	= "Beast Mastery"
local n_Hunter_talent_Marksmanship	= "Marksmanship"
local n_Hunter_talent_Survival      = "Survival"

local n_Mage_talent_Arcane          = "Arcane"
local n_Mage_talent_Fire            = "Fire"
local n_Mage_talent_Frost           = "Frost"

local n_Warrior_talent_Arms         = "Arms"
local n_Warrior_talent_Fury         = "Fury"
local n_Warrior_talent_Protection   = "Protection"

local n_Warlock_talent_Affliction   = "Affliction"
local n_Warlock_talent_Demonology   = "Demonology"
local n_Warlock_talent_Destruction  = "Destruction"

local n_Rogue_talent_Assasination   = "Assassination"
local n_Rogue_talent_Combat         = "Combat"
local n_Rogue_talent_Subtlety       = "Subtlety"

local n_Priest_talent_Discipline    = "Discipline"
local n_Priest_talent_Holy          = "Holy"
local n_Priest_talent_Shadow        = "Shadow"

--------------------------------------------------------------------------
-- Dark Intent Priority Lists
--------------------------------------------------------------------------
-- Priority based on Gherkin's Dark Intent thread:
-- http://www.mmo-champion.com/threads/813759-Mechanics-Dark-Intent-(4.2)

local RaidDpsGainDestruction = {
   { n_PRIEST,        n_Priest_talent_Shadow	    		},
   { n_DRUID,         n_Druid_talent_Balance         	},
   { n_MAGE,          n_Mage_talent_Fire             	},
   { n_DRUID,					n_Druid_talent_Restoration			},
   { n_SHAMAN,				n_Shaman_talent_Restoration			},
   { n_SHAMAN,        n_Shaman_talent_Elemental     	},
   { n_MAGE,          n_Mage_talent_Frost             },
   { n_MAGE,          n_Mage_talent_Arcane            },
   { n_DRUID,         n_Druid_talent_FeralCombat     	},
   { n_HUNTER,        n_Hunter_talent_Survival        },
   { n_PALADIN,       n_Paladin_talent_Retribution    },
   { n_ROGUE,         n_Rogue_talent_Combat           },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	2	},    -- (2H)
   { n_ROGUE,         n_Rogue_talent_Assasination     },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		2	},    -- (2H)
   { n_HUNTER,        n_Hunter_talent_Marksmanship    },
   { n_HUNTER,        n_Hunter_talent_BeastMastery  	},
   { n_SHAMAN,        n_Shaman_talent_Enhancement    	},
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		1	},    -- (1H)
   { n_DEATHKNIGHT,		n_DeathKnight_talent_Unholy,	1	},    -- (1H)
}

local RaidDpsGainAfflictionDrainLife = {
   { n_PRIEST,        n_Priest_talent_Shadow    			},
   { n_DRUID,         n_Druid_talent_Balance         	},
   { n_MAGE,          n_Mage_talent_Fire             	},
   { n_DRUID,					n_Druid_talent_Restoration			},
   { n_SHAMAN,				n_Shaman_talent_Restoration			},
   { n_SHAMAN,        n_Shaman_talent_Elemental     	},
   { n_MAGE,          n_Mage_talent_Frost             },
   { n_MAGE,          n_Mage_talent_Arcane            },
   { n_DRUID,         n_Druid_talent_FeralCombat     	},
   { n_HUNTER,        n_Hunter_talent_Survival        },
   { n_PALADIN,       n_Paladin_talent_Retribution    },
   { n_ROGUE,         n_Rogue_talent_Combat           },
   { n_HUNTER,        n_Hunter_talent_Marksmanship    },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	2	},    -- (2H)
   { n_SHAMAN,        n_Shaman_talent_Enhancement    	},
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	1	},    -- (1H)
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		2	},    -- (2H)
   { n_ROGUE,         n_Rogue_talent_Assasination     },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		1	},    -- (1H)
   { n_HUNTER,				n_Hunter_talent_BeastMastery  	},
}

local RaidDpsGainAfflictionShadowBolt = {
   { n_PRIEST,        n_Priest_talent_Shadow    			},
   { n_DRUID,         n_Druid_talent_Balance         	},
   { n_MAGE,          n_Mage_talent_Fire             	},
   { n_DRUID,					n_Druid_talent_Restoration			},
   { n_SHAMAN,				n_Shaman_talent_Restoration			},
   { n_SHAMAN,        n_Shaman_talent_Elemental     	},
   { n_MAGE,          n_Mage_talent_Frost             },
   { n_MAGE,          n_Mage_talent_Arcane            },
   { n_DRUID,         n_Druid_talent_FeralCombat     	},
   { n_HUNTER,        n_Hunter_talent_Survival        },
   { n_PALADIN,       n_Paladin_talent_Retribution    },
   { n_ROGUE,         n_Rogue_talent_Combat           },
   { n_SHAMAN,        n_Shaman_talent_Enhancement    	},
   { n_HUNTER,        n_Hunter_talent_Marksmanship    },
   { n_ROGUE,         n_Rogue_talent_Assasination     },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		1	},    -- (1H)
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		2	},    -- (2H)
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	2	},    -- (2H)
   { n_HUNTER,        n_Hunter_talent_BeastMastery  	},
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	1	},    -- (1H)
}

local RaidDpsGainDemonology = {
   { n_PRIEST,        n_Priest_talent_Shadow    			},
   { n_DRUID,         n_Druid_talent_Balance         	},
   { n_MAGE,          n_Mage_talent_Fire             	},
   { n_DRUID,					n_Druid_talent_Restoration			},
   { n_SHAMAN,				n_Shaman_talent_Restoration			},
   { n_SHAMAN,        n_Shaman_talent_Elemental     	},
   { n_MAGE,          n_Mage_talent_Frost             },
   { n_MAGE,          n_Mage_talent_Arcane            },
   { n_DRUID,         n_Druid_talent_FeralCombat     	},
   { n_HUNTER,        n_Hunter_talent_Survival        },
   { n_PALADIN,       n_Paladin_talent_Retribution    },
   { n_ROGUE,         n_Rogue_talent_Combat           },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy,	2	},    -- (2H)
   { n_ROGUE,         n_Rogue_talent_Assasination     },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		2	},    -- (2H)
   { n_HUNTER,        n_Hunter_talent_Marksmanship    },
   { n_HUNTER,        n_Hunter_talent_BeastMastery  	},
   { n_SHAMAN,        n_Shaman_talent_Enhancement    	},
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost,		1	},    -- (1H)
   { n_DEATHKNIGHT,		n_DeathKnight_talent_Unholy,	1	},    -- (1H)
}

local PersonalUptime = {
   { n_DRUID,         n_Druid_talent_Balance					},
   { n_PRIEST,        n_Priest_talent_Shadow      		},
   { n_DRUID,					n_Druid_talent_Restoration			},
   { n_SHAMAN,				n_Shaman_talent_Restoration			},
   { n_HUNTER,        n_Hunter_talent_Survival    		},
   { n_DRUID,         n_Druid_talent_FeralCombat  		},
   { n_MAGE,          n_Mage_talent_Fire          		},
   { n_MAGE,          n_Mage_talent_Frost         		},
   { n_ROGUE,         n_Rogue_talent_Assasination 		},
   { n_SHAMAN,        n_Shaman_talent_Elemental   		},
   { n_DEATHKNIGHT,		n_DeathKnight_talent_Unholy,	1	},    -- (1H)
   { n_HUNTER,        n_Hunter_talent_BeastMastery		},
   { n_ROGUE,         n_Rogue_talent_Combat           },
   { n_MAGE,          n_Mage_talent_Arcane            },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Unholy, 	2 },    -- (2H)
   { n_HUNTER,        n_Hunter_talent_Marksmanship    },
   { n_SHAMAN,        n_Shaman_talent_Enhancement    	},
   { n_PALADIN,       n_Paladin_talent_Retribution    },
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost, 	2 },    -- (2H)
   { n_DEATHKNIGHT,   n_DeathKnight_talent_Frost, 	1 },    -- (1H)
}


local	ef = CreateFrame("Frame", nil, UIParent)
local oorRecheck = 5
local oorElapsed = 0


local groupdb = {}
local groupType = "party"
local numMembers = 1
local prilist
local currChoiceName
local prevChoiceName
local oorQueue = {}				-- Out of Range party members
local inspectQueue = {}		-- In range members awaiting inspection
local inspectInProgress = false

-----------------------------------------------------------
-- UpdatePriList
-----------------------------------------------------------
function edm:UpdatePriList()
	print("UpdatePriList()")
   
  --set up for party or raid

	if groupType == "raid" then   
	  --see what kind of warlock I am
	  local myspec = LGT:GetUnitTalentSpec("player")    
  	--print("I am a " .. myspec .. " Warlock")

  	-- pick priority list based on current spec
		if myspec == "Destruction" then
			prilist = RaidDpsGainDestruction
		elseif myspec == "Demonology" then
			prilist = RaidDpsGainDemonolgy
		elseif myspec == "Affliction" then
		  local ptsBane = LGT:UnitHasTalent("player", "Bane")
			if ptsBane == 3 then
				-- Bane 3/3 = Shadow Bolt
 				prilist = RaidDpsGainShadowBolt
			else
				-- Bane 0/3 = Drain Life
				prilist = RaidDpsGainAfflictionDrainLife
			end
		end
	else
   prilist = PersonalUptime
	end
end

-----------------------------------------------------------
-- SelectDarkIntentTarget
-----------------------------------------------------------
function edm:SelectDarkIntentTarget()
  --print("SelectDarkIntentTarget()")

  local choiceName  = nil
  local choiceRank  = 100
  local choiceRealm = nil
  local WeaponType  = 0

  if not prilist then
  	prilist = PersonalUptime
  end   
   
  for k, v in pairs(groupdb) do
    
    --print("k: " .. k .. ", class: " .. v.class .. " spec: " .. v.spec)
    
		for i = 1, #prilist do
			if i < choiceRank and v.class == prilist[i][1] and v.spec == prilist[i][2] then
      	if v.class ~= n_DEATHKNIGHT then
        	print(v.name .. " is number " .. i .. " on the " .. groupType .. " priority list")
          choiceRank = i
          choiceName = v.name
          choiceRealm = v.realm
        else	-- it's a DK
					if v.spec ~= n_DeathKnight_talent_Blood then
			    	print("Getting mainhand item for Death Knight.")
			      local weaponName, _, _, _, _, _, subclass = GetItemInfo(v.mainhandID)
			      print(v.name .. " is wielding a " .. weaponName .. ", which is a " .. subclass)
			      if strmatch(subclass, "One") then
			        print("subclass = One")
			        WeaponType = 1
			      elseif strmatch(subclass, "Two") then
			        print("subclass = Two")
			        WeaponType = 2
			      end
        
	          if WeaponType == prilist[i][3] then
  	          print(v.name .. " is number " .. i .. " on the " .. groupType .. " priority list")
      	      choiceRank = i
        	    choiceName = v.name
          	  choiceRealm = v.realm
          	end
          end
        end
      end
    end -- i, #prilist do
	end -- k,v in pairs(groupdb)
	
  if choiceName then   
    -- build up name  for cross realm whisper if necessary
    if choiceRealm  then
      currChoiceName = choiceName .. "-" .. choiceRealm:gsub("%s", "")
    else
      currChoiceName = choiceName
    end
    --  whisper the recipient
    if  currChoiceName ~= prevChoiceName then
      local msg = "Hello " .. choiceName .. ", Please stay near while I cast Dark Intent."
      SendChatMessage(msg, "WHISPER", nil, currChoiceName)
      prevChoiceName = currChoiceName
    end
  else
    print("All of our choices are really bad.")
  end
end -- SelectDarkIntentTarget()

-----------------------------------------------------------
-- RequestNextInspect
-----------------------------------------------------------
function edm:RequestNextInspect()
  --print("RequestNextInspect()")
  local m = table.remove(inspectQueue)
  if m then
    inspectInProgress = true
    NotifyInspect(m)
  else
    print("All inspections are complete")
      edm:SelectDarkIntentTarget()
    --end   
  end   
end

-----------------------------------------------------------
-- HandleInspectReady
-----------------------------------------------------------
function edm:INSPECT_READY(guid)
  --print("INSPECT_READY(", guid, ")")
  if groupdb[guid] then
	  print("Inspect ready for " .. groupdb[guid].name)
    groupdb[guid].inspected = true
    groupdb[guid].mainhandID = GetInventoryItemID(groupdb[guid].groupID, INVSLOT_MAINHAND)
    local name, _, _, _, _, _, subclass = GetItemInfo(groupdb[guid].mainhandID)
    ClearInspectPlayer()
    inspectInProgress = false
  end
  self:RequestNextInspect()
end

-----------------------------------------------------------
-- CheckOutOfRangeQueue
-----------------------------------------------------------
function edm:CheckOutOfRangeQueue()

	for i = 1, #oorQueue do
		local m = table.remove(oorQueue)
    if CheckInteractDistance(m, 1) then
	   	if not inspectInProgress then
  	 		inspectInProgress = true
     		NotifyInspect(m)
     	else
       	table.insert(inspectQueue, m)
      end
    else
    	table.insert(oorQueue, m)
    end
	end    
	
	if #oorQueue == 0 then
		ef:SetScript("OnUpdate", nil)
	end
end

-----------------------------------------------------------
-- FrameOnUpdate
-----------------------------------------------------------
local function FrameOnUpdate(self, elapsed)
   totalElapsed = totalElapsed + elapsed
   if totalElapsed >= oorRecheck then
   	totalElapsed = totalElapsed - oorRecheck
   	edm:CheckOutOfRangeQueue()
   end
end

-----------------------------------------------------------
-- UpdateGroupMemberData
-----------------------------------------------------------
function edm:UpdateGroupMemberData()
  print("UpdateGroupMemberData()")
   for i = 1, numMembers do
      local m = groupType .. i
      if UnitExists(m) and UnitInRaid(m) or UnitInParty(m) then
         local guid = UnitGUID(m)
         if not groupdb[guid] then
            --print("Obtaining information for " .. m)
            local name, realm = UnitName(m)
            local lclass, class = UnitClass(m)
				    local spec = LGT:GetUnitTalentSpec(m)
            
            print("guid: " .. guid .. ", name: " .. name .. ", spec: " .. spec .. ", class: " .. class)

            groupdb[guid] = {}
            groupdb[guid].groupID = m
            groupdb[guid].name = name
            groupdb[guid].realm = realm
            groupdb[guid].lclass = lclass
            groupdb[guid].class = class
            groupdb[guid].spec = spec
            groupdb[guid].inspected = true

            if class == n_DEATHKNIGHT and spec ~= n_DeathKnight_talent_Blood then
	          	print(name .. " is a " .. spec .. " Death Knight, requesting inspect...")
	            groupdb[guid].inspected = false
              if CheckInteractDistance(m, 1) then
              	if not inspectInProgress then
              		inspectInProgress = true
              		NotifyInspect(m)
              	else
      		       	table.insert(inspectQueue, m)
      		      end
              else
              	table.insert(oorQueue, m)
             		ef:SetScript("OnUpdate", FrameOnUpdate)
              end
            end
         end
      end
   end
end

-----------------------------------------------------------
-- PARTY_MEMBERS_CHANGED
-----------------------------------------------------------
function edm:PARTY_MEMBERS_CHANGED()
  print("PARTY_MEMBERS_CHANGED()")

   if UnitInRaid("player") then
      numMembers = GetNumRaidMembers()
      groupType = "raid"
   elseif UnitInParty("player") then
      numMembers = GetNumPartyMembers() + 1
   else
      currChoiceName = nil
      prevChoiceName = nil
      numMembers = 0
      groupType = "none"
      groupdb = {}
   end

  if numMembers > 1 then
    print("There are " .. numMembers .. " in the " .. groupType .. ".") 
    self:UpdatePriList()
    self:UpdateGroupMemberData()
  else
  	-- reset everything
    print("I am all alone.")
		currChoiceName = nil
		prevChoiceName = nil
  	groupdb = {}
  end
end

-----------------------------------------------------------
-- RAID_ROSTER_UPDATE
-----------------------------------------------------------
function edm:RAID_ROSTER_UPDATE()
  self:PARTY_MEMBERS_CHANGED()
end

-----------------------------------------------------------
-- FrameOnEvent
-----------------------------------------------------------
local function FrameOnEvent(this, event, ...)
		print("FrameOnEvent(" .. event .. ")")
    edm[event](edm, ...)
end

-----------------------------------------------------------
-- CreateFrame
-----------------------------------------------------------
function edm:InitFrame()
	print("InitFrame()")
	ef:RegisterEvent("INSPECT_READY")
	ef:RegisterEvent("PARTY_MEMBERS_CHANGED")
	ef:RegisterEvent("RAID__ROSTER_UPDATE")
	ef:SetScript("OnEvent", FrameOnEvent)
end

-----------------------------------------------------------
-- LibGroupTalents_UpdateComplete
-----------------------------------------------------------
function edm:LibGroupTalents_UpdateComplete()
  print("LibGroupTalents_UpdateComplete()")
  self:PARTY_MEMBERS_CHANGED()
  self:UpdateGroupMemberData()
  if numMembers > 1 and not inspectInProgess and #inspectQueue == 0 then
  	self:SelectDarkIntentTarget()
  end
end

-----------------------------------------------------------
-- Initialize
-----------------------------------------------------------
function edm:Initialize()
	self:InitFrame()
	LGT.RegisterCallback(edm, "LibGroupTalents_UpdateComplete")
	print("Endarken Me version " .. edm.major .. "." .. edm.minor .. "." .. edm.build .. " loaded")
end

-----------------------------------------------------------
-- Kick it all off here
-----------------------------------------------------------
edm:Initialize()
